
From: Andi Kleen <ak@muc.de>

From: James Cleverdon

The value that cpuinfo returns for command 1 in ebx is the physical APIC ID
latched when the system comes out of reset.

Ordinarily, this is identical to the value in the local APIC's ID register,
because nearly all BIOSes accept the HW assigned value.

Our systems, made up of individual building blocks, can't do that.  Each
node boots as a separate system and is joined together by the BIOS.  Thus,
the BIOS rewrites the local APIC ID register with a new value.

Potomac and Nocona chips have a mechanism by which the BIOS writer can
change bits 7:5 to match the assigned cluster ID.  Bits 2:0 come from the
thread ID.  However, bits 4:3 are still those latched at reset.  Oops!

Summary: Large clustered systems can't use cpuid to derive the sibling
information.

Fix: Use the local APIC ID.  That's the value we use to online the CPUs, so
it had better be OK.  For non-clustered systems, cpuid == local APIC, so
nothing but large boxes should be affected.

Signed-off-by: Andi Kleen <ak@suse.de>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/x86_64/kernel/setup.c |    2 +-
 1 files changed, 1 insertion(+), 1 deletion(-)

diff -puN arch/x86_64/kernel/setup.c~x86-64-sibling-map-fix-for-clustered-mode arch/x86_64/kernel/setup.c
--- 25/arch/x86_64/kernel/setup.c~x86-64-sibling-map-fix-for-clustered-mode	Tue Sep 14 17:03:55 2004
+++ 25-akpm/arch/x86_64/kernel/setup.c	Tue Sep 14 17:03:55 2004
@@ -745,7 +745,7 @@ static void __init detect_ht(struct cpui
 		}
 		if (index_lsb != index_msb )
 			index_msb++;
-		initial_apic_id = ebx >> 24 & 0xff;
+		initial_apic_id = hard_smp_processor_id();
 		phys_proc_id[cpu] = initial_apic_id >> index_msb;
 		
 		printk(KERN_INFO  "CPU: Physical Processor ID: %d\n",
_
